@@ -1,16 +1,13 @@
 from functools import wraps
 
 from flask import Blueprint, render_template, url_for, request, redirect, session
-from piecash import open_book
+from flask import current_app as app
 from sqlalchemy.exc import OperationalError
 
-from flask import current_app as app
+from .utils.gnucash import open_book, AccessDenied
 
 bp = Blueprint('auth', __name__, url_prefix='/auth')
 
-class AccessDenied(Exception):
-    pass
-
 @bp.app_errorhandler(AccessDenied)
 def handle_account_not_found(e: AccessDenied):
     end_session()
@@ -30,7 +27,9 @@ def authenticate(username, password):
         return True
     elif app.config.AUTH_MECHANISM == 'passthrough':
         try:
-            with open_book(uri_conn=app.config.DB_URI(username, password)) as book:
+            # Check authn by attempting to connect to database
+            with open_book(uri_conn=app.config.DB_URI(username, password),
+                           readonly=True, open_if_lock=True) as book:
                 app.logger.debug(f'Authentication succeeded for {username}')
                 session['username'] = username
                 session['password'] = password
@@ -62,7 +61,7 @@ def requires_auth(func):
         if is_authenticated():
             return func(*args, **kwargs)
         else:
-            return redirect(url_for('auth.login'))
+            return redirect(url_for('auth.login', return_url=request.url))
 
     return inner
 
@@ -71,11 +70,13 @@ def requires_auth(func):
 def login():
     if request.method == 'POST':
        authenticate(request.form['username'], request.form['password'])
-
-    if is_authenticated():
-        return render_template('user.j2', username=session.get('username', 'no one'))
+       session.permanent = True
+       return redirect(request.args.get('return_url') or url_for('.login'))
     else:
-        return render_template('login.j2')
+        if is_authenticated():
+            return render_template('user.j2', username=session.get('username', 'no one'))
+        else:
+            return render_template('login.j2')
 
 @bp.route('/logout', methods=['POST'])
 def logout():
